﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;

public class NewIssueAvailableEventArgs : EventArgs
{
    public string hyperlinkURL;
    public string issueNumber;
    public DateTime datePublished;
}

[ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession)]
public class MagazineService : IMagazineService
{
    public static event NewIssueAvailableHandler NewIssueAvailableEvent;
    public delegate void NewIssueAvailableHandler(object sender, NewIssueAvailableEventArgs e);

    // instance of eventhandler
    NewIssueAvailableHandler newIssueHandler;

    // Callback to individual clients using the ClientContract
    IClientContract callback = null;

    public void PublishMagazine(string hyperLinkToIssue, string issueNumber, DateTime datePublished)
    {
        NewIssueAvailableEventArgs eargs = new NewIssueAvailableEventArgs();
        eargs.hyperlinkURL = hyperLinkToIssue;
        eargs.issueNumber = issueNumber;
        eargs.datePublished = datePublished;

        // fire the event
        NewIssueAvailableEvent(this, eargs);
    }

    public void Subscribe()
    {
        callback = OperationContext.Current.GetCallbackChannel<IClientContract>();
        // this is the handler that we will 'unsubscribe' from later
        newIssueHandler = new NewIssueAvailableHandler(MagazineService_NewIssueAvailableEvent);
        NewIssueAvailableEvent += newIssueHandler;
    }

    public void Unsubscribe()
    {
        NewIssueAvailableEvent -= newIssueHandler;
    }

    public void MagazineService_NewIssueAvailableEvent(object sender, NewIssueAvailableEventArgs e)
    {
        callback.MessageReceived(e.hyperlinkURL, e.issueNumber, e.datePublished);
    }
}


